<?php

namespace App\Http\Controllers;

use App\Helper\Files;
use App\Helper\Reply;
use App\Models\ProjectFile;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Storage;
use Symfony\Component\HttpFoundation\Response;

class ProjectFileController extends AccountBaseController
{

    /**
     * @param Request $request
     * @return mixed|void
     * @throws \Froiden\RestAPI\Exceptions\RelatedResourceNotFoundException
     */
    public function store(Request $request)
    {

        if ($request->hasFile('file')) {

            $this->storeFiles($request);

            $this->files = ProjectFile::where('project_id', $request->project_id)->orderByDesc('id')->get();

            $view = view('projects.files.show', $this->data)->render();

            return Reply::dataOnly(['status' => 'success', 'view' => $view]);
        }
    }

    public function storeMultiple(Request $request)
    {
        if ($request->hasFile('file')) {
            $this->storeFiles($request);
        }
    }

    private function storeFiles($request)
    {
        foreach ($request->file as $fileData) {

            $file = new ProjectFile();
            $file->project_id = $request->project_id;
            $file->service_id = $request->service_id;
            $file->case_id = $request->case_id;

            //$filename = Files::uploadLocalOrS3($fileData, ProjectFile::FILE_PATH . '/' . $request->project_id);
            $filename = Files::uploadLocalOrS3($fileData, ProjectFile::FILE_PATH . '/' . $request->project_id);

            //dd($fileData);
            /*$filename = Files::uploadLocalOrS3Test($fileData, ProjectFile::AWS_FILE_PATH);*/

            $file->user_id = $this->user->id;
            $file->filename = $fileData->getClientOriginalName();
            $file->hashname = $filename;
            $file->size = $fileData->getSize();
            $file->save();
            $this->logProjectActivity($request->project_id, 'messages.newFileUploadedToTheProject');
        }
    }

    // public function destroy(Request $request, $id)
    // {
    //     $file = ProjectFile::findOrFail($id);
    //     $deleteDocumentPermission = user()->permission('delete_project_files');
    //     abort_403(!($deleteDocumentPermission == 'all' || ($deleteDocumentPermission == 'added' && $file->added_by == user()->id)));

    //     Files::deleteFile($file->hashname, ProjectFile::FILE_PATH . '/' . $file->project_id);

    //     ProjectFile::destroy($id);

    //     $this->files = ProjectFile::where('project_id', $file->project_id)->orderByDesc('id')->get();

    //     $view = view('projects.files.show', $this->data)->render();

    //     return Reply::successWithData(__('messages.deleteSuccess'), ['view' => $view]);
    // }

    public function destroy(Request $request, $id)
{
    $file = ProjectFile::findOrFail($id);
    $deleteDocumentPermission = user()->permission('delete_project_files');
    abort_403(!($deleteDocumentPermission == 'all' || ($deleteDocumentPermission == 'added' && $file->added_by == user()->id)));

    // Check if the file exists in S3 and delete it
    if (Storage::disk('s3')->exists('projectfiles/' . $file->hashname)) {
        Storage::disk('s3')->delete('projectfiles/' . $file->hashname);
    }

    // Delete the file from the local storage if needed
    Files::deleteFile($file->hashname, ProjectFile::FILE_PATH . '/' . $file->project_id);

    // Delete the file record from the database
    ProjectFile::destroy($id);

    // Refresh the list of files for the project
    $this->files = ProjectFile::where('project_id', $file->project_id)->orderByDesc('id')->get();

    $view = view('projects.files.show', $this->data)->render();

    return Reply::successWithData(__('messages.deleteSuccess'), ['view' => $view]);
}


    public function download($id)
    {
        $file = ProjectFile::whereRaw('md5(id) = ?', $id)->firstOrFail();
        $this->viewPermission = user()->permission('view_project_files');
        abort_403(!($this->viewPermission == 'all' || ($this->viewPermission == 'added' && $file->added_by == user()->id)));

        return download_local_s3($file, ProjectFile::FILE_PATH . '/' . $file->project_id . '/' . $file->hashname);

    }

}
