<?php

namespace App\Models;

use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;
use DB;

class CaseService extends Model
{
    use HasFactory;

    /**
     * The database table used by the model.
     *
     * @var string
     */
    protected $table = 'case_services';


    /**
     * The attributes that are mass assignable.
     *
     * @var array
     */
    protected $fillable = [
        'case_id',
        'guest_id',
        'user_id',
        'services_ids',
        'created_at',
        'updated_at',        
    ];


    /**
     * Method is used to search news detail.
     *
     * @param array $search
     * @param int $skip
     * @param int $perPage
     *
     * @return mixed
     */
    public function getCaseServices($search = null, $skip, $perPage)
    {
        $take = ((int)$perPage > 0) ? $perPage : 20;
        // default filter if no search
        $filter = 1;

        $fields = [
            "case_services.*", 
            DB::raw("CONCAT(case_client_information.first_name, ' ', case_client_information.last_name) as full_name"),
            "case_client_information.dob",
            "case_client_information.email",
            "case_client_information.mobile_no"                   
         ];

        /*if (is_array($search) && count($search) > 0) {
            $keyword = (array_key_exists('keyword', $search)) ? " AND name LIKE '%" .
                addslashes(trim($search['keyword'])) . "%' " : "";

            $filter .= $keyword;
        }*/        

        $result =  $this->leftJoin('case_client_information', 'case_services.case_id', '=', 'case_client_information.case_id')
                        ->whereRaw($filter)
                        ->orderBy('case_services.id', 'DESC')
                        ->skip($skip)
                        ->take($take)
                        ->get($fields);

        return $result;
    }

    /**
     * Method is used to get total category search wise.
     *
     * @param array $search
     *
     * @return mixed
     */
    public function totalCaseServices($search = null)
    {
        // if no search add where
        $filter = 1;

       /* if (is_array($search) && count($search) > 0) {
            $keyword = (array_key_exists('keyword', $search)) ? " AND name LIKE '%" .
                addslashes(trim($search['keyword'])) . "%' " : "";
            $filter .= $keyword;
        }*/

        return $this
            ->select(\DB::raw('count(*) as total'))
            ->whereRaw($filter)
            ->first();       

    }

    public function getCaseClientInformation($case_id)
    {
        $fields = [
            "case_services.*", 
            DB::raw("CONCAT(case_client_information.first_name, ' ', case_client_information.last_name) as full_name"),
            "case_client_information.saluation",
            "case_client_information.dob",
            "case_client_information.email",
            "case_client_information.mobile_no",
            "case_client_information.address",
         ];

        //dd($fields);             

         return $this->join('case_client_information', 'case_services.case_id', '=', 'case_client_information.case_id')
                    ->where('case_services.case_id',$case_id)
                    ->first($fields);
    }

    public function getServiceByCase($case_id)
    {
        $data = $this->where('case_id', $case_id)->first(['services_ids']);
        if(!empty($data->services_ids)){
           return $data->services_ids; 
        }
        return null;        
    }

    public function getCaseByCaseNumber($case_number){

        $data = $this->where('case_id', $case_number)->first(['case_id']);
        if(!empty($data->case_id)){
           return $data->case_id; 
        }
        return null;
    } 

    public function getProjectCaseByProjectID($project_id){

        $data = $this->join('projects', 'case_services.case_id', '=', 'projects.case_id')->where('projects.id', $project_id)->first(['case_services.*']);
        if(!empty($data)){
           return $data; 
        }
        return null;
    }

}
